"""
Unit tests for tools.py
"""
import unittest
from os import path
import yaml
from exsource_tools import load_exsource_file, ExSource
from exsource_tools import tools

TEST_PATH = path.dirname(__file__)

#pylint: disable=protected-access

# Docstrings have little value in the unit tests
#pylint: disable=missing-class-docstring
#pylint: disable=missing-function-docstring

class ExSourceTestCase(unittest.TestCase):

    def test_roundtrip(self):
        example_file = path.join(TEST_PATH, "examples", "readme_example.yml")
        with open(example_file, 'r', encoding="utf-8") as file_obj:
            input_data = yaml.safe_load(file_obj.read())
        exsource = load_exsource_file(example_file)
        self.assertEqual(exsource.dump(), input_data)

    def test_output_multiply_defined(self):
        example_file = path.join(TEST_PATH, "examples", "readme_example.yml")
        with open(example_file, 'r', encoding="utf-8") as file_obj:
            input_data = yaml.safe_load(file_obj.read())
        input_data["exports"]["another_frame"] = {"output-files": ["frame.step"],
                                                  "source-files": ["frame2.FCStd"],
                                                  "application": "freecad"}
        with self.assertLogs('exsource', level='WARN'):
            ExSource(input_data)

    def test_file_format(self):
        yaml_exts = ["exsource.yaml",
                     "exsource.yml",
                     "exsource-def.yaml",
                     "yaml.yml",
                     "json.yaml"]
        json_exts = ["exsource.json",
                     "exsource-def.json",
                     "yaml.json",
                     "json.json"]
        other_exts = ["exsource",
                      "exsource-def",
                      "exsource.xml",
                      "yaml",
                      "json"]
        for filename in yaml_exts:
            self.assertEqual(tools._exsource_file_format(filename), "YAML")
        for filename in json_exts:
            self.assertEqual(tools._exsource_file_format(filename), "JSON")
        for filename in other_exts:
            with self.assertRaises(ValueError) as context:
                tools._exsource_file_format(filename)
            self.assertTrue(f"Couldn't read '{filename}'." in str(context.exception))
